@@ -23,11 +23,12 @@
 #include <stdexcept>
 #include <vector>
 
-#include "../rocFFT/shared/fft_params.h"
+#include "../../shared/fft_params.h"
 
-#include "accuracy_test.h"
-#include "fftw_transform.h"
-#include "rocfft_against_fftw.h"
+#include "../../shared/accuracy_test.h"
+#include "../../shared/fftw_transform.h"
+#include "../../shared/params_gen.h"
+#include "../../shared/rocfft_against_fftw.h"
 
 using ::testing::ValuesIn;
 
@@ -115,7 +116,8 @@ const static std::vector<std::vector<size_t>> ooffset_range = {{0, 0}, {1, 1}};
 
 INSTANTIATE_TEST_SUITE_P(pow2_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow2_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow2_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -128,7 +130,8 @@ INSTANTIATE_TEST_SUITE_P(pow2_1D,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow2_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow2_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -142,7 +145,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D,
 
 INSTANTIATE_TEST_SUITE_P(pow2_1D_half,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow2_range_half}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow2_range_half}),
                                                              {fft_precision_half},
                                                              batch_range_1D,
                                                              stride_range,
@@ -155,7 +159,8 @@ INSTANTIATE_TEST_SUITE_P(pow2_1D_half,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D_half,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow2_range_half}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow2_range_half}),
                                                              {fft_precision_half},
                                                              batch_range_1D,
                                                              stride_range,
@@ -169,7 +174,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow2_1D_half,
 
 INSTANTIATE_TEST_SUITE_P(pow3_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow3_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow3_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -182,7 +188,8 @@ INSTANTIATE_TEST_SUITE_P(pow3_1D,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow3_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow3_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow3_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -196,7 +203,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow3_1D,
 
 INSTANTIATE_TEST_SUITE_P(pow5_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow5_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow5_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -209,7 +217,8 @@ INSTANTIATE_TEST_SUITE_P(pow5_1D,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow5_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({pow5_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({pow5_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -223,7 +232,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_pow5_1D,
 
 INSTANTIATE_TEST_SUITE_P(radX_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({radX_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({radX_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -236,7 +246,8 @@ INSTANTIATE_TEST_SUITE_P(radX_1D,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_radX_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({radX_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({radX_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -250,7 +261,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_radX_1D,
 
 INSTANTIATE_TEST_SUITE_P(prime_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({prime_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({prime_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -263,7 +275,8 @@ INSTANTIATE_TEST_SUITE_P(prime_1D,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_prime_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({prime_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({prime_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -277,7 +290,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_prime_1D,
 
 INSTANTIATE_TEST_SUITE_P(mix_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({mix_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({mix_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -290,7 +304,8 @@ INSTANTIATE_TEST_SUITE_P(mix_1D,
                          accuracy_test::TestName);
 INSTANTIATE_TEST_SUITE_P(DISABLED_offset_mix_1D,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator(generate_lengths({mix_range}),
+                         ::testing::ValuesIn(param_generator(test_prob,
+                                                             generate_lengths({mix_range}),
                                                              precision_range_sp_dp,
                                                              batch_range_1D,
                                                              stride_range,
@@ -307,6 +322,7 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_offset_mix_1D,
 INSTANTIATE_TEST_SUITE_P(small_1D,
                          accuracy_test,
                          ::testing::ValuesIn(param_generator_base(
+                             test_prob,
                              {fft_transform_type_complex_forward},
                              generate_lengths({small_1D_sizes()}),
                              {fft_precision_single},
@@ -345,7 +361,8 @@ const static std::vector<size_t>              batch_range_for_stride     = {2, 1
 INSTANTIATE_TEST_SUITE_P(
     pow2_1D_stride_complex,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_complex(generate_lengths({pow2_range_for_stride}),
+    ::testing::ValuesIn(param_generator_complex(test_prob,
+                                                generate_lengths({pow2_range_for_stride}),
                                                 precision_range_sp_dp,
                                                 batch_range_1D,
                                                 stride_range_for_pow2,
@@ -360,7 +377,8 @@ INSTANTIATE_TEST_SUITE_P(
 INSTANTIATE_TEST_SUITE_P(
     pow2_1D_stride_real,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_real(generate_lengths({pow2_range_for_stride}),
+    ::testing::ValuesIn(param_generator_real(test_prob,
+                                             generate_lengths({pow2_range_for_stride}),
                                              precision_range_sp_dp,
                                              batch_range_1D,
                                              stride_range_for_pow2,
@@ -375,7 +393,8 @@ INSTANTIATE_TEST_SUITE_P(
 INSTANTIATE_TEST_SUITE_P(
     pow2_1D_stride_real_half,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_real(generate_lengths({pow2_range_for_stride_half}),
+    ::testing::ValuesIn(param_generator_real(test_prob,
+                                             generate_lengths({pow2_range_for_stride_half}),
                                              {fft_precision_half},
                                              batch_range_1D,
                                              stride_range_for_pow2,
@@ -389,7 +408,8 @@ INSTANTIATE_TEST_SUITE_P(
 
 // Create an array parameters for strided 2D batched transforms.
 inline auto
-    param_generator_complex_1d_batched_2d(const std::vector<std::vector<size_t>>&  v_lengths,
+    param_generator_complex_1d_batched_2d(const double                             base_prob,
+                                          const std::vector<std::vector<size_t>>&  v_lengths,
                                           const std::vector<fft_precision>&        precision_range,
                                           const std::vector<std::vector<size_t>>&  ioffset_range,
                                           const std::vector<std::vector<size_t>>&  ooffset_range,
@@ -437,7 +457,27 @@ inline auto
                             param.ioffset        = ioffset;
                             param.ooffset        = ooffset;
 
-                            params.push_back(param);
+                            param.validate();
+
+                            const double roll = hash_prob(random_seed, param.token());
+                            const double run_prob
+                                = base_prob * (param.is_planar() ? complex_planar_prob_factor : 1.0)
+                                  * (param.is_interleaved() ? complex_interleaved_prob_factor : 1.0)
+                                  * (param.is_real() ? real_prob_factor : 1.0);
+
+                            if(roll > run_prob)
+                            {
+                                if(verbose > 4)
+                                {
+                                    std::cout << "Test skipped (probability " << run_prob << " > "
+                                              << roll << ")\n";
+                                }
+                                continue;
+                            }
+                            if(param.valid(0))
+                            {
+                                params.push_back(param);
+                            }
                         }
                     }
                 }
@@ -453,7 +493,8 @@ const static std::vector<size_t> pow2_range_2D
 INSTANTIATE_TEST_SUITE_P(
     pow2_1D_complex_batched_2D_strided,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({pow2_range_2D}),
+    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(test_prob,
+                                                              generate_lengths({pow2_range_2D}),
                                                               precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
@@ -464,7 +505,8 @@ const static std::vector<size_t> pow3_range_2D = {3, 27, 81, 243, 729, 2187, 656
 INSTANTIATE_TEST_SUITE_P(
     pow3_1D_complex_batched_2D_strided,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({pow3_range_2D}),
+    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(test_prob,
+                                                              generate_lengths({pow3_range_2D}),
                                                               precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
@@ -475,7 +517,8 @@ const static std::vector<size_t> pow5_range_2D = {5, 25, 125, 625, 3125, 15625};
 INSTANTIATE_TEST_SUITE_P(
     pow5_1D_complex_batched_2D_strided,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({pow5_range_2D}),
+    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(test_prob,
+                                                              generate_lengths({pow5_range_2D}),
                                                               precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
@@ -487,7 +530,8 @@ const static std::vector<size_t> prime_range_2D = {7, 11, 13, 17, 19, 23, 29, 26
 INSTANTIATE_TEST_SUITE_P(
     prime_1D_complex_batched_2D_strided,
     accuracy_test,
-    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(generate_lengths({prime_range_2D}),
+    ::testing::ValuesIn(param_generator_complex_1d_batched_2d(test_prob,
+                                                              generate_lengths({prime_range_2D}),
                                                               precision_range_sp_dp,
                                                               ioffset_range_zero,
                                                               ooffset_range_zero,
