@@ -18,7 +18,8 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "accuracy_test.h"
+#include "../../shared/accuracy_test.h"
+#include "../../shared/params_gen.h"
 
 std::vector<std::vector<size_t>> callback_sizes = {
     // some single kernel sizes
@@ -105,7 +106,8 @@ auto transform_types = {fft_transform_type_complex_forward, fft_transform_type_r
 #ifdef __HIP__
 INSTANTIATE_TEST_SUITE_P(callback,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator_base(transform_types,
+                         ::testing::ValuesIn(param_generator_base(test_prob,
+                                                                  transform_types,
                                                                   callback_sizes,
                                                                   precision_range_sp_dp,
                                                                   batch_range,
@@ -121,7 +123,8 @@ INSTANTIATE_TEST_SUITE_P(callback,
 
 INSTANTIATE_TEST_SUITE_P(DISABLED_callback,
                          accuracy_test,
-                         ::testing::ValuesIn(param_generator_base(transform_types,
+                         ::testing::ValuesIn(param_generator_base(test_prob,
+                                                                  transform_types,
                                                                   callback_sizes,
                                                                   precision_range_sp_dp,
                                                                   batch_range,
@@ -141,7 +144,8 @@ INSTANTIATE_TEST_SUITE_P(DISABLED_callback,
 // result scaling feature.
 inline auto param_generator_scaling(const std::vector<std::vector<size_t>>& v_lengths)
 {
-    auto params = param_generator(callback_sizes,
+    auto params = param_generator(test_prob,
+                                  callback_sizes,
                                   precision_range_sp_dp,
                                   batch_range,
                                   stride_range,
