@@ -1,4 +1,4 @@
-// Copyright (C) 2022 - 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (C) 2022 - 2023 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -19,6 +19,7 @@
 // THE SOFTWARE.
 
 #include <boost/scope_exit.hpp>
+#include <boost/tokenizer.hpp>
 #include <gtest/gtest.h>
 #include <hip/hip_runtime.h>
 #include <math.h>
@@ -26,23 +27,33 @@
 #include <utility>
 #include <vector>
 
-#include "hipfft.h"
+#include "hipfft/hipfft.h"
 
 #include "../hipfft_params.h"
-#include "../rocFFT/clients/tests/fftw_transform.h"
-#include "../rocFFT/clients/tests/rocfft_accuracy_test.h"
-#include "../rocFFT/clients/tests/rocfft_against_fftw.h"
-#include "../rocFFT/shared/gpubuf.h"
 
-void fft_vs_reference(hipfft_params& params)
+#include "../../shared/accuracy_test.h"
+#include "../../shared/fftw_transform.h"
+#include "../../shared/gpubuf.h"
+#include "../../shared/rocfft_against_fftw.h"
+#include "../../shared/rocfft_complex.h"
+#include "../../shared/subprocess.h"
+
+extern std::string mp_launch;
+
+extern last_cpu_fft_cache last_cpu_fft_data;
+
+void fft_vs_reference(hipfft_params& params, bool round_trip)
 {
     switch(params.precision)
     {
+    case fft_precision_half:
+        fft_vs_reference_impl<rocfft_fp16, hipfft_params>(params, round_trip);
+        break;
     case fft_precision_single:
-        fft_vs_reference_impl<float, hipfft_params>(params);
+        fft_vs_reference_impl<float, hipfft_params>(params, round_trip);
         break;
     case fft_precision_double:
-        fft_vs_reference_impl<double, hipfft_params>(params);
+        fft_vs_reference_impl<double, hipfft_params>(params, round_trip);
         break;
     }
 }
@@ -63,44 +74,73 @@ TEST_P(accuracy_test, vs_fftw)
         GTEST_SKIP();
     }
 
-    fft_vs_reference(params);
+    switch(params.mp_lib)
+    {
+    case fft_params::fft_mp_lib_none:
+    {
+        // only do round trip for single-GPU FFTs
+        bool round_trip = params.multiGPU <= 1;
+
+        if(!params.run_callbacks)
+        {
+            try
+            {
+                fft_vs_reference(params, round_trip);
+            }
+            catch(HOSTBUF_MEM_USAGE& e)
+            {
+                // explicitly clear cache
+                last_cpu_fft_data = last_cpu_fft_cache();
+                GTEST_SKIP() << e.msg;
+            }
+            catch(ROCFFT_SKIP& e)
+            {
+                GTEST_SKIP() << e.msg;
+            }
+            catch(ROCFFT_FAIL& e)
+            {
+                GTEST_FAIL() << e.msg;
+            }
+        }
+        break;
+    }
+    case fft_params::fft_mp_lib_mpi:
+    {
+        // split launcher into tokens since the first one is the exe
+        // and the remainder is the start of its argv
+        boost::escaped_list_separator<char>                   sep('\\', ' ', '\"');
+        boost::tokenizer<boost::escaped_list_separator<char>> tokenizer(mp_launch, sep);
+        std::string                                           exe;
+        std::vector<std::string>                              argv;
+        for(auto t : tokenizer)
+        {
+            if(t.empty())
+                continue;
+
+            if(exe.empty())
+                exe = t;
+            else
+                argv.push_back(t);
+        }
+        // append test token and ask for accuracy test
+        argv.push_back("--token");
+        argv.push_back(params.token());
+        argv.push_back("--accuracy");
+
+        // throws an exception if launch fails or if subprocess
+        // returns nonzero exit code
+        execute_subprocess(exe, argv, {});
+        break;
+    }
+    default:
+        GTEST_FAIL() << "Invalid communicator choice!";
+        break;
+    }
+
     SUCCEED();
 }
 
 #ifdef __HIP__
-__host__ __device__ float multiply_by_scalar(float a, double b)
-{
-    return a * b;
-}
-__host__ __device__ float2 multiply_by_scalar(float2 a, double b)
-{
-    return hipCmulf(a, make_float2(b, 0.0));
-}
-__host__ __device__ double multiply_by_scalar(double a, double b)
-{
-    return a * b;
-}
-__host__ __device__ double2 multiply_by_scalar(double2 a, double b)
-{
-    return hipCmul(a, make_double2(b, 0.0));
-}
-
-__host__ __device__ float add_scalar(float a, double b)
-{
-    return a + b;
-}
-__host__ __device__ float2 add_scalar(float2 a, double b)
-{
-    return hipCaddf(a, make_float2(b, 0.0));
-}
-__host__ __device__ double add_scalar(double a, double b)
-{
-    return a + b;
-}
-__host__ __device__ double2 add_scalar(double2 a, double b)
-{
-    return hipCadd(a, make_double2(b, 0.0));
-}
 
 // load/store callbacks - cbdata in each is actually a scalar double
 // with a number to apply to each element
@@ -110,7 +150,7 @@ __host__ __device__ Tdata load_callback(Tdata* input, size_t offset, void* cbdat
     auto testdata = static_cast<const callback_test_data*>(cbdata);
     // multiply each element by scalar
     if(input == testdata->base)
-        return multiply_by_scalar(input[offset], testdata->scalar);
+        return input[offset] * testdata->scalar;
     // wrong base address passed, return something obviously wrong
     else
     {
@@ -119,12 +159,16 @@ __host__ __device__ Tdata load_callback(Tdata* input, size_t offset, void* cbdat
     }
 }
 
-__device__ auto load_callback_dev_float   = load_callback<float>;
-__device__ auto load_callback_dev_float2  = load_callback<float2>;
-__device__ auto load_callback_dev_double  = load_callback<double>;
-__device__ auto load_callback_dev_double2 = load_callback<double2>;
+__device__ auto load_callback_dev_half           = load_callback<rocfft_fp16>;
+__device__ auto load_callback_dev_complex_half   = load_callback<rocfft_complex<rocfft_fp16>>;
+__device__ auto load_callback_dev_float          = load_callback<float>;
+__device__ auto load_callback_dev_complex_float  = load_callback<rocfft_complex<float>>;
+__device__ auto load_callback_dev_double         = load_callback<double>;
+__device__ auto load_callback_dev_complex_double = load_callback<rocfft_complex<double>>;
 
-void* get_load_callback_host(fft_array_type itype, fft_precision precision)
+void* get_load_callback_host(fft_array_type itype,
+                             fft_precision  precision,
+                             bool           round_trip_inverse = false)
 {
     void* load_callback_host = nullptr;
     switch(itype)
@@ -134,14 +178,21 @@ void* get_load_callback_host(fft_array_type itype, fft_precision precision)
     {
         switch(precision)
         {
+        case fft_precision_half:
+            EXPECT_EQ(hipMemcpyFromSymbol(&load_callback_host,
+                                          HIP_SYMBOL(load_callback_dev_complex_half),
+                                          sizeof(void*)),
+                      hipSuccess);
+            return load_callback_host;
         case fft_precision_single:
-            EXPECT_EQ(hipMemcpyFromSymbol(
-                          &load_callback_host, HIP_SYMBOL(load_callback_dev_float2), sizeof(void*)),
+            EXPECT_EQ(hipMemcpyFromSymbol(&load_callback_host,
+                                          HIP_SYMBOL(load_callback_dev_complex_float),
+                                          sizeof(void*)),
                       hipSuccess);
             return load_callback_host;
         case fft_precision_double:
             EXPECT_EQ(hipMemcpyFromSymbol(&load_callback_host,
-                                          HIP_SYMBOL(load_callback_dev_double2),
+                                          HIP_SYMBOL(load_callback_dev_complex_double),
                                           sizeof(void*)),
                       hipSuccess);
             return load_callback_host;
@@ -151,6 +202,11 @@ void* get_load_callback_host(fft_array_type itype, fft_precision precision)
     {
         switch(precision)
         {
+        case fft_precision_half:
+            EXPECT_EQ(hipMemcpyFromSymbol(
+                          &load_callback_host, HIP_SYMBOL(load_callback_dev_half), sizeof(void*)),
+                      hipSuccess);
+            return load_callback_host;
         case fft_precision_single:
             EXPECT_EQ(hipMemcpyFromSymbol(
                           &load_callback_host, HIP_SYMBOL(load_callback_dev_float), sizeof(void*)),
@@ -177,17 +233,21 @@ __host__ __device__ static void
     // add scalar to each element
     if(output == testdata->base)
     {
-        output[offset] = add_scalar(element, testdata->scalar);
+        output[offset] = element + testdata->scalar;
     }
     // otherwise, wrong base address passed, just don't write
 }
 
-__device__ auto store_callback_dev_float   = store_callback<float>;
-__device__ auto store_callback_dev_float2  = store_callback<float2>;
-__device__ auto store_callback_dev_double  = store_callback<double>;
-__device__ auto store_callback_dev_double2 = store_callback<double2>;
+__device__ auto store_callback_dev_half           = store_callback<rocfft_fp16>;
+__device__ auto store_callback_dev_complex_half   = store_callback<rocfft_complex<rocfft_fp16>>;
+__device__ auto store_callback_dev_float          = store_callback<float>;
+__device__ auto store_callback_dev_complex_float  = store_callback<rocfft_complex<float>>;
+__device__ auto store_callback_dev_double         = store_callback<double>;
+__device__ auto store_callback_dev_complex_double = store_callback<rocfft_complex<double>>;
 
-void* get_store_callback_host(fft_array_type otype, fft_precision precision)
+void* get_store_callback_host(fft_array_type otype,
+                              fft_precision  precision,
+                              bool           round_trip_inverse = false)
 {
     void* store_callback_host = nullptr;
     switch(otype)
@@ -197,15 +257,21 @@ void* get_store_callback_host(fft_array_type otype, fft_precision precision)
     {
         switch(precision)
         {
+        case fft_precision_half:
+            EXPECT_EQ(hipMemcpyFromSymbol(&store_callback_host,
+                                          HIP_SYMBOL(store_callback_dev_complex_half),
+                                          sizeof(void*)),
+                      hipSuccess);
+            return store_callback_host;
         case fft_precision_single:
             EXPECT_EQ(hipMemcpyFromSymbol(&store_callback_host,
-                                          HIP_SYMBOL(store_callback_dev_float2),
+                                          HIP_SYMBOL(store_callback_dev_complex_float),
                                           sizeof(void*)),
                       hipSuccess);
             return store_callback_host;
         case fft_precision_double:
             EXPECT_EQ(hipMemcpyFromSymbol(&store_callback_host,
-                                          HIP_SYMBOL(store_callback_dev_double2),
+                                          HIP_SYMBOL(store_callback_dev_complex_double),
                                           sizeof(void*)),
                       hipSuccess);
             return store_callback_host;
@@ -215,6 +281,11 @@ void* get_store_callback_host(fft_array_type otype, fft_precision precision)
     {
         switch(precision)
         {
+        case fft_precision_half:
+            EXPECT_EQ(hipMemcpyFromSymbol(
+                          &store_callback_host, HIP_SYMBOL(store_callback_dev_half), sizeof(void*)),
+                      hipSuccess);
+            return store_callback_host;
         case fft_precision_single:
             EXPECT_EQ(hipMemcpyFromSymbol(&store_callback_host,
                                           HIP_SYMBOL(store_callback_dev_float),
@@ -236,7 +307,7 @@ void* get_store_callback_host(fft_array_type otype, fft_precision precision)
 }
 
 // implement result scaling as a store callback, as rocFFT tests do
-void apply_store_callback(const fft_params& params, fftw_data_t& output)
+void apply_store_callback(const fft_params& params, std::vector<hostbuf>& output)
 {
     if(!params.run_callbacks && params.scale_factor == 1.0)
         return;
@@ -252,12 +323,29 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
     {
         switch(params.precision)
         {
+        case fft_precision_half:
+        {
+            const size_t elem_size = sizeof(std::complex<rocfft_fp16>);
+            const size_t num_elems = output.front().size() / elem_size;
+
+            auto output_begin
+                = reinterpret_cast<rocfft_complex<rocfft_fp16>*>(output.front().data());
+            for(size_t i = 0; i < num_elems; ++i)
+            {
+                auto& element = output_begin[i];
+                if(params.scale_factor != 1.0)
+                    element = element * params.scale_factor;
+                if(params.run_callbacks)
+                    store_callback(output_begin, i, element, &cbdata, nullptr);
+            }
+            break;
+        }
         case fft_precision_single:
         {
             const size_t elem_size = sizeof(std::complex<float>);
             const size_t num_elems = output.front().size() / elem_size;
 
-            auto output_begin = reinterpret_cast<float2*>(output.front().data());
+            auto output_begin = reinterpret_cast<rocfft_complex<float>*>(output.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 auto& element = output_begin[i];
@@ -273,7 +361,7 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
             const size_t elem_size = sizeof(std::complex<double>);
             const size_t num_elems = output.front().size() / elem_size;
 
-            auto output_begin = reinterpret_cast<double2*>(output.front().data());
+            auto output_begin = reinterpret_cast<rocfft_complex<double>*>(output.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 auto& element = output_begin[i];
@@ -293,6 +381,23 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
         // planar wouldn't run callbacks, but we could still want scaling
         switch(params.precision)
         {
+        case fft_precision_half:
+        {
+            const size_t elem_size = sizeof(std::complex<rocfft_fp16>);
+            for(auto& buf : output)
+            {
+                const size_t num_elems = buf.size() / elem_size;
+
+                auto output_begin = reinterpret_cast<rocfft_complex<rocfft_fp16>*>(buf.data());
+                for(size_t i = 0; i < num_elems; ++i)
+                {
+                    auto& element = output_begin[i];
+                    if(params.scale_factor != 1.0)
+                        element = element * params.scale_factor;
+                }
+            }
+            break;
+        }
         case fft_precision_single:
         {
             const size_t elem_size = sizeof(std::complex<float>);
@@ -300,7 +405,7 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
             {
                 const size_t num_elems = buf.size() / elem_size;
 
-                auto output_begin = reinterpret_cast<float2*>(buf.data());
+                auto output_begin = reinterpret_cast<rocfft_complex<float>*>(buf.data());
                 for(size_t i = 0; i < num_elems; ++i)
                 {
                     auto& element = output_begin[i];
@@ -317,7 +422,7 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
             {
                 const size_t num_elems = buf.size() / elem_size;
 
-                auto output_begin = reinterpret_cast<double2*>(buf.data());
+                auto output_begin = reinterpret_cast<rocfft_complex<double>*>(buf.data());
                 for(size_t i = 0; i < num_elems; ++i)
                 {
                     auto& element = output_begin[i];
@@ -334,6 +439,22 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
     {
         switch(params.precision)
         {
+        case fft_precision_half:
+        {
+            const size_t elem_size = sizeof(rocfft_fp16);
+            const size_t num_elems = output.front().size() / elem_size;
+
+            auto output_begin = reinterpret_cast<rocfft_fp16*>(output.front().data());
+            for(size_t i = 0; i < num_elems; ++i)
+            {
+                auto& element = output_begin[i];
+                if(params.scale_factor != 1.0)
+                    element = element * params.scale_factor;
+                if(params.run_callbacks)
+                    store_callback(output_begin, i, element, &cbdata, nullptr);
+            }
+            break;
+        }
         case fft_precision_single:
         {
             const size_t elem_size = sizeof(float);
@@ -376,7 +497,7 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
 }
 
 // apply load callback if necessary
-void apply_load_callback(const fft_params& params, fftw_data_t& input)
+void apply_load_callback(const fft_params& params, std::vector<hostbuf>& input)
 {
     if(!params.run_callbacks)
         return;
@@ -394,12 +515,24 @@ void apply_load_callback(const fft_params& params, fftw_data_t& input)
     {
         switch(params.precision)
         {
+        case fft_precision_half:
+        {
+            const size_t elem_size = sizeof(std::complex<rocfft_fp16>);
+            const size_t num_elems = input.front().size() / elem_size;
+
+            auto input_begin = reinterpret_cast<rocfft_complex<rocfft_fp16>*>(input.front().data());
+            for(size_t i = 0; i < num_elems; ++i)
+            {
+                input_begin[i] = load_callback(input_begin, i, &cbdata, nullptr);
+            }
+            break;
+        }
         case fft_precision_single:
         {
             const size_t elem_size = sizeof(std::complex<float>);
             const size_t num_elems = input.front().size() / elem_size;
 
-            auto input_begin = reinterpret_cast<float2*>(input.front().data());
+            auto input_begin = reinterpret_cast<rocfft_complex<float>*>(input.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 input_begin[i] = load_callback(input_begin, i, &cbdata, nullptr);
@@ -411,7 +544,7 @@ void apply_load_callback(const fft_params& params, fftw_data_t& input)
             const size_t elem_size = sizeof(std::complex<double>);
             const size_t num_elems = input.front().size() / elem_size;
 
-            auto input_begin = reinterpret_cast<double2*>(input.front().data());
+            auto input_begin = reinterpret_cast<rocfft_complex<double>*>(input.front().data());
             for(size_t i = 0; i < num_elems; ++i)
             {
                 input_begin[i] = load_callback(input_begin, i, &cbdata, nullptr);
@@ -425,6 +558,18 @@ void apply_load_callback(const fft_params& params, fftw_data_t& input)
     {
         switch(params.precision)
         {
+        case fft_precision_half:
+        {
+            const size_t elem_size = sizeof(rocfft_fp16);
+            const size_t num_elems = input.front().size() / elem_size;
+
+            auto input_begin = reinterpret_cast<rocfft_fp16*>(input.front().data());
+            for(size_t i = 0; i < num_elems; ++i)
+            {
+                input_begin[i] = load_callback(input_begin, i, &cbdata, nullptr);
+            }
+            break;
+        }
         case fft_precision_single:
         {
             const size_t elem_size = sizeof(float);
@@ -462,19 +607,36 @@ void apply_load_callback(const fft_params& params, fftw_data_t& input)
 // Many seem to be called unconditionally, so we can't throw exceptions in
 // most cases.
 
-void* get_load_callback_host(fft_array_type itype, fft_precision precision)
+void* get_load_callback_host(fft_array_type itype,
+                             fft_precision  precision,
+                             bool           round_trip_inverse = false)
 {
     return nullptr;
 }
-void apply_load_callback(const fft_params& params, fftw_data_t& input) {}
+void apply_load_callback(const fft_params& params, std::vector<hostbuf>& input) {}
 
 // implement result scaling as a store callback, as rocFFT tests do
-void apply_store_callback(const fft_params& params, fftw_data_t& output)
+void apply_store_callback(const fft_params& params, std::vector<hostbuf>& output)
 {
     if(params.scale_factor == 1.0)
         return;
     switch(params.precision)
     {
+    case fft_precision_half:
+    {
+        const size_t elem_size = sizeof(rocfft_fp16);
+        for(auto& buf : output)
+        {
+            const size_t num_elems    = buf.size() / elem_size;
+            auto         output_begin = reinterpret_cast<rocfft_fp16*>(buf.data());
+            for(size_t i = 0; i < num_elems; ++i)
+            {
+                auto& element = output_begin[i];
+                element       = static_cast<double>(element) * params.scale_factor;
+            }
+        }
+        break;
+    }
     case fft_precision_single:
     {
         const size_t elem_size = sizeof(float);
@@ -507,9 +669,11 @@ void apply_store_callback(const fft_params& params, fftw_data_t& output)
     }
     }
 }
-void* get_store_callback_host(fft_array_type otype, fft_precision precision)
+void* get_store_callback_host(fft_array_type otype,
+                              fft_precision  precision,
+                              bool           round_trip_inverse = false)
 {
     throw std::runtime_error("get_store_callback_host not implemented");
     return nullptr;
 }
-#endif
\ No newline at end of file
+#endif
