@@ -23,8 +23,8 @@
 #ifndef ROCFFT_ACCURACY_TEST
 #define ROCFFT_ACCURACY_TEST
 
+#include "../../shared/accuracy_test.h"
 #include "../hipfft_params.h"
-#include "../rocFFT/clients/tests/accuracy_test.h"
 
 void fft_vs_reference(hipfft_params& params, bool round_trip = false);
 
