@@ -19,7 +19,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-#include "hipfft.h"
+#include "hipfft/hipfft.h"
 #include <fftw3.h>
 #include <gtest/gtest.h>
 #include <hip/hip_vector_types.h>
@@ -181,7 +181,6 @@ TEST(hipfftTest, CreatePlanMany64)
     long long int const nY                 = 128;
     long long int const nZ                 = 23;
     long long int       n[3]               = {nX, nY, nZ};
-    long long int       n_invalid[3]       = {nX, -nY, nZ};
     long long int       inembed[3]         = {nX, nY, nZ};
     long long int const istride            = 1;
     long long int const idist              = nX * nY * nZ;
@@ -218,7 +217,10 @@ TEST(hipfftTest, CreatePlanMany64)
         ASSERT_EQ(hipfftDestroy(plan_valid), HIPFFT_SUCCESS);
     }
 
-    // invalid data layout (n array has a negative entry)
+    // invalid data layout (n array has a negative entry). only test rocFFT
+    // backend, since it's more strict
+#ifdef __HIP_PLATFORM_AMD__
+    long long int n_invalid[3] = {nX, -nY, nZ};
     {
         hipfftHandle plan_invalid_1 = hipfft_params::INVALID_PLAN_HANDLE;
         ASSERT_EQ(hipfftCreate(&plan_invalid_1), HIPFFT_SUCCESS);
@@ -238,6 +240,7 @@ TEST(hipfftTest, CreatePlanMany64)
         ASSERT_EQ(hipfftSetAutoAllocation(plan_invalid_1, 0), HIPFFT_SUCCESS);
         ASSERT_EQ(hipfftDestroy(plan_invalid_1), HIPFFT_SUCCESS);
     }
+#endif
 
     // invalid data layout (onembed array has a negative entry)
     {
@@ -374,7 +377,10 @@ TEST(hipfftTest, CheckBufferSizeR2C)
 {
     hipfftHandle plan = hipfft_params::INVALID_PLAN_HANDLE;
     ASSERT_EQ(hipfftCreate(&plan), HIPFFT_SUCCESS);
-    size_t n        = 2048;
+    // real forward transform cannot modify input, so we need to pick
+    // a sufficiently small N such that rocFFT can fuse
+    // post-processing into one kernel and avoid a temp buffer
+    size_t n        = 256;
     size_t workSize = 0;
 
     ASSERT_EQ(hipfftMakePlan1d(plan, n, HIPFFT_R2C, 1, &workSize), HIPFFT_SUCCESS);
@@ -424,7 +430,10 @@ TEST(hipfftTest, CheckBufferSizeD2Z)
 {
     hipfftHandle plan = hipfft_params::INVALID_PLAN_HANDLE;
     ASSERT_EQ(hipfftCreate(&plan), HIPFFT_SUCCESS);
-    size_t n        = 2048;
+    // real forward transform cannot modify input, so we need to pick
+    // a sufficiently small N such that rocFFT can fuse
+    // post-processing into one kernel and avoid a temp buffer
+    size_t n        = 256;
     size_t batch    = 1000;
     size_t workSize = 0;
 
@@ -554,6 +563,7 @@ TEST(hipfftTest, RunR2C)
     nrmse /= maxv;
 
     EXPECT_TRUE(nrmse < type_epsilon<double>());
+    fftw_destroy_plan(ref_p);
     fftw_free(ref_out);
 }
 
@@ -564,9 +574,12 @@ TEST(hipfftTest, RunR2C)
 // the out-place plan from working.
 TEST(hipfftTest, OutplaceOnly)
 {
-    int   N_in  = 4;
-    int   N_out = N_in / 2 + 1;
-    float in[N_in];
+    static const int N_in_const  = 4;
+    static const int N_out_const = N_in_const / 2 + 1;
+    // mutable sizes for passing to hipFFT
+    int   N_in  = N_in_const;
+    int   N_out = N_out_const;
+    float in[N_in_const];
     for(int i = 0; i < N_in; i++)
         in[i] = i + (i % 3) - (i % 7);
 
@@ -600,8 +613,8 @@ TEST(hipfftTest, OutplaceOnly)
     ASSERT_EQ(hipFree(d_in), hipSuccess);
     ASSERT_EQ(hipFree(d_out), hipSuccess);
 
-    double ref_in[N_in];
-    for(int i = 0; i < N_in; i++)
+    double ref_in[N_in_const];
+    for(int i = 0; i < N_in_const; i++)
         ref_in[i] = in[i];
 
     fftw_complex* ref_out;
@@ -628,5 +641,6 @@ TEST(hipfftTest, OutplaceOnly)
     nrmse /= maxv;
 
     ASSERT_TRUE(nrmse < type_epsilon<double>());
+    fftw_destroy_plan(ref_p);
     fftw_free(ref_out);
 }
